'use strict';

var objCase = require('obj-case');
var extend = require('@ndhoule/extend');
var values = require('@ndhoule/values');
var pick = require('@ndhoule/pick');
var each = require('@ndhoule/each');
var md5 = require('md5');
var isEmail = require('is-email');
var useHttps = require('use-https');

/**
 * Module dependencies.
 */

var integration = require('@segment/analytics.js-integration');

/**
 * Expose `INTEGRATION_NAME` integration.
 */

var Criteo = module.exports = integration('Criteo')
  .option('account', '')
  .option('homeUrl', '')
  .option('supportingUserData', {})
  .option('supportingPageData', {})
  .tag('http', '<script src="http://static.criteo.net/js/ld/ld.js">')
  .tag('https', '<script src="https://static.criteo.net/js/ld/ld.js">');

/**
 * Initialize.
 *
 * @api public
 */

Criteo.prototype.initialize = function() {
  var account = this.options.account;

  window.criteo_q = window.criteo_q || [];
  window.criteo_q.push({ event: 'setAccount', account: account });

  var protocol = useHttps() ? 'https': 'http';
  this.load(protocol, this.ready);
};

/**
 * Loaded?
 *
 * @api public
 * @return {boolean}
 */

Criteo.prototype.loaded = function() {
  // what are required properties or functions that you need available on the `window`
  // before the integration marks itself as ready?
  return !!(window.criteo_q && window.criteo_q.push !== Array.prototype.push);
};

/**
 * Page
 *
 * @api public
 */

Criteo.prototype.page = function(page) {
  var supportingPageData = values(this.options.supportingPageData);
  var properties = page.properties(this.options.supportingPageData);
  var extraData = pick(supportingPageData, properties);
  var homeUrl = this.options.homeUrl;
  var pageName = (page.name() || '').toLowerCase();
  var path = page.path();
  var url = page.url();

  window.criteo_q.push(extend(extraData, { event: 'setData' }));

  if (url === homeUrl || pageName === 'home' || path === '/') {
    window.criteo_q.push({ event: 'viewHome' });
  }
};

/**
 * Product Viewed
 *
 * @api public
 */

Criteo.prototype.productViewed = function(track) {
  var productId = track.productId() || '';
  var payload = { event: 'viewItem', product: productId };

  setData.call(this);

  window.criteo_q.push(payload);
};

/**
 * Product List Viewed
 *
 * @api public
 */

Criteo.prototype.productListViewed = function(track) {
  var products = track.products() || [];
  var productIds = [];

  each(function(product) {
    var id = objCase.find(product, 'productId');
    if (id) productIds.push(id);
  }, products);

  var payload = { event: 'viewList', product: productIds };

  setData.call(this);

  window.criteo_q.push(payload);
};

/**
 * Cart Viewed
 *
 * @api public
 */

Criteo.prototype.cartViewed = function(track) {
  var products = getProductMetadata(track);
  var payload = { event: 'viewBasket', product: products };

  setData.call(this);

  window.criteo_q.push(payload);
};

/**
 * Order Completed
 *
 * @api public
 */

Criteo.prototype.orderCompleted = function(track) {
  var orderId = objCase.find(track.properties(), 'orderId');
  var products = getProductMetadata(track);
  var payload = {
    event: 'trackTransaction',
    id: orderId || '',
    product: products
  };

  setData.call(this);

  window.criteo_q.push(payload);
};

/**
 * Get metadata from events with products array
 *
 * @api private
 */

function getProductMetadata(track) {
  var products = track.proxy('properties.products') || [];

  products = products.map(function(product) {
    var id = objCase.find(product, 'productId') || '';
    var price = product.price || '';
    var quantity = product.quantity || '';

    return { id: id, price: price, quantity: quantity };
  });

  return products;
}

/**
 * Add extra data to each tag payload.
 * Extra data is defined as an integration setting.
 *
 */

function setData() {
  var extraData = {};
  // Add userId if available as customer_id
  var userId = this.analytics.user().id();

  // Criteo does NOT want emails passed as customer_id.
  if (userId && !isEmail(userId)) {
    window.criteo_q.push({ event: 'setCustomerId', id: userId });
  }

  // Check cached traits for any that have been defined as extraData params.
  var traits = this.analytics.user().traits();

  // Criteo has a special tag for emails.
  // They also require all emails get passed as md5 hashes.
  if (traits.email) {
    window.criteo_q.push({ event: 'setEmail', email: md5(traits.email) });
    delete traits.email;
  }

  var supportingUserData = this.options.supportingUserData;

  each(function(value, key) {
    var trait = objCase.find(traits, key);

    if (trait) extraData[value] = trait;
  }, supportingUserData);

  // { event: "setData", ui_membership: "free", ui_age: "30"}
  window.criteo_q.push(extend(extraData, { event: 'setData' }));
}
